#include <iostream>
#include <vector>
#include <random>
#include <algorithm>


using namespace std;

static int n, m;

std::vector<bool> genHint(const std::vector<int>& aa, const std::vector<int>& bb, const std::vector<int>& sol) {
    auto a = aa, b = bb;
    if (a.size() > b.size()) {
        swap(a, b);
    } // a.size() <= b.size()
    vector<bool> hint;
    if (a.size() > 1e4) {
        int j = 0;
        hint.assign(a.size(), false);
        for (int i = 0; i < a.size() && j < sol.size(); ++i) {
            if (a[i] == sol[j]) {
                hint[i] = true;
                ++j;
            }
        }
    }
    return hint;
}

vector<int> kvadrat(vector<int> a, vector<int> b) {
    vector<vector<int>> dp(a.size() + 1, vector<int>(b.size() + 1, 0));
    for (int i = 0; i < a.size(); ++i) {
        for (int j = 0; j < b.size(); ++j) {
            if (a[i] == b[j]) {
                dp[i + 1][j + 1] = 1 + dp[i][j];
            } else {
                dp[i + 1][j + 1] = max(dp[i][j + 1], dp[i + 1][j]);
            }
        }
    }
    vector<int> res;
    int i = a.size(), j = b.size();
    while (i > 0 && j > 0) {
        if (dp[i][j] > max(dp[i - 1][j], dp[i][j - 1])) {
            res.push_back(a[i - 1]);
            --i;
            --j;
        } else if (dp[i - 1][j] == dp[i][j]) {
            --i;
        } else {
            --j;
        }
    }
    reverse(res.begin(), res.end());
    return res;
}

std::vector<int> solve(const std::vector<int>& aa, const std::vector<int>& bb, const std::vector<bool>& hint) {
    auto a = aa, b = bb;
    if (a.size() > b.size()) {
        swap(a, b);
    } // a.size() <= b.size()
    if (a.size() <= 1e4) {
        return kvadrat(a, b);
    }
    vector<int> res;
    if (a.size() < b.size()) {
        for (int i = 0; i < a.size(); ++i) {
            if (hint[i]) {
                res.push_back(a[i]);
            }
        }
    } else {
        for (int i = 0; i < b.size(); ++i) {
            if (hint[i]) {
                res.push_back(b[i]);
            }
        }
    }
    return res;
}

vector<int> brute(vector<int> a, vector<int> b) {
    if (a.size() > b.size()) {
        swap(a, b);
    }
    vector<int> res;
    for (int mask = 0; mask < (1 << a.size()); ++mask) {
        vector<int> currRes;
        for (int i = 0; i < a.size(); ++i) {
            if ((mask >> i) & 1) {
                currRes.push_back(a[i]);
            }
        }
        int j = 0;
        for (int i = 0; i < b.size() && j < currRes.size(); ++i) {
            if (b[i] == currRes[j]) {
                ++j;
            }
        }
        if (j == currRes.size() && currRes.size() > res.size()) {
            res = currRes;
        }
    }
    return res;
}

void print(vector<int>& a) {
    cout << a.size() << '\n';
    for (auto i : a) {
        cout << i << ' ';
    }
    cout << "\n\n";
}

int main()
{
    mt19937 mt(time(nullptr));
    std::vector<int> a, b, sol, maybeSol;
    vector<bool> hint;
    n = mt() % 10 + 2;
    m = mt() % 1000 + 2;

    a.resize(n);
    b.resize(m);

    for (int& num : a)
    {
        num = mt() % int(10);
//        std::cin >> num;
    }

    for (int& num : b)
    {
        num = mt() % int(10);
        //std::cin >> num;
    }

    //std::cin >> k;
    //k = mt() % 100
    //ol.resize(k);

//    for (int& num : sol)
//    {
//        std::cin >> num;
//    }

    sol = brute(a, b);
    cout << n << ' ' << sol.size() << '\n';
//    print(a);
//    print(b);
//    print(sol);
    hint = genHint(a, b, sol);
    maybeSol = solve(a, b, hint);

    std::cout << hint.size() << std::endl;
    std::cout << maybeSol.size() << std::endl;
    if (maybeSol == sol) {
        cout << "OK\n";
        return 0;
    } else {
        cout << "BAN\n";
    }
    for (int i = 0; i < (int) maybeSol.size(); ++i)
    {
        if (i > 0) std::cout << " ";
        std::cout << maybeSol[i];
    }
    std::cout << std::endl;

    return 0;
}
