#include <iostream>
#include <vector>
#include <algorithm>
#include <cassert>
#include <cmath>
#include "hint.h"

int const sigma = 640;
int n, bucketsize;

std::vector<bool>  genHint(const  std::vector<int>&  a,  const  std::vector<int>&  b, 
    const std::vector<int>& sol) {
  int n = a.size();
  int m = b.size();
  bucketsize = std::max(n, m) / (sigma / 2) + 1;
  int start = 0, stop = 0;
  int ptr = 0, ptr2 = 0;

  std::vector<bool> hint;

  for(int i = 0; i < sol.size(); i++) {
    while(a[ptr] != sol[i])
      ptr++;
    while(b[ptr2] != sol[i])
      ptr2++;
    

    while((start + 1) * bucketsize <= ptr) {
      start++;
      hint.push_back(0);
    }
    while((stop + 1) * bucketsize <= ptr2) {
      stop++;
      hint.push_back(1);
    }
  }
  assert(hint.size() <= 640);
  return hint;
}

int const gigamax = 200000000;
int last[1 + gigamax];

std::vector<int> startx, starty, stopx, stopy;
std::vector<int> startid;
std::vector<std::pair<int,int>>  dp, dp2;

inline int getid(int base, int x, int y) {
  return (x - startx[base]) * (stopy[base] - starty[base] + 1) + y - starty[base];
}

std::pair<int,int> extract(int base, int x, int y) {
  if(startx[base] <= x && starty[base] <= y) {
    int where = getid(base, x, y);
    return {dp2[where].first, dp2[where].second};
  } else if(0 == base) {
    return {0, 0};
  } else {
    base--;
    x = std::min(x, stopx[base]);
    y = std::min(y, stopy[base]);
    if(x < 0 || y < 0)
      return {0, 0};
    if(startx[base] <= x && x <= stopx[base] && starty[base] <= y && y <= stopy[base] ){
      int where = getid(base, x, y);
      return {dp[where].first, dp[where].second};
    } else
      return {0, 0};
  }
}

std::vector<int> solve(const std::vector<int>& a, const std::vector<int>& b, 
    const std::vector<bool>& hint) {

  int n = a.size();
  int m = b.size();

  bucketsize = std::max(n, m) / (sigma / 2) + 1;
  int k = hint.size() + 1;
  startx = std::vector<int>(k, 0), starty = std::vector<int>(k, 0), stopx = std::vector<int>(k, 0), stopy = std::vector<int>(k, 0);
  startid = std::vector<int>(k, 0);

  int bsqr = bucketsize * bucketsize;

  dp = std::vector<std::pair<int,int>>(1 + bsqr, {0, 0});
  dp2 = std::vector<std::pair<int,int>>(1 + bsqr, {0, 0});
  std::vector<std::pair<int,int>> global(k, {0, 0});

  for(int i = 0; i < k; i++) {
    if(0 < i) {
      if(hint[i - 1] == 0) {
        startx[i] = startx[i - 1] + bucketsize;
        starty[i] = starty[i - 1];
      } else {
        startx[i] = startx[i - 1];
        starty[i] = starty[i - 1] + bucketsize;
      }
      startid[i] = startid[i - 1] + (stopx[i - 1] - startx[i - 1] + 1) * (stopy[i - 1] - starty[i - 1] + 1);
    }
    stopx[i] = std::min(n - 1, startx[i] + bucketsize - 1);
    stopy[i] = std::min(m - 1, starty[i] + bucketsize - 1);
  }

  int smax = 0, smaxid = 0;
  
  //std::cerr << "helper: " << k << '\n';
  
  int ptrglobal = -1;
  for(int i = 0; i < k; i++) {
    int ptr = 0;  
    if(0 < i)
      global[i] = global[i - 1];
    while(stopx[ptrglobal + 1] < startx[i] && stopy[ptrglobal + 1] < starty[i])
      ptrglobal++;


    for(int j = startx[i]; j <= stopx[i]; j++)
      for(int j2 = starty[i]; j2 <= stopy[i]; j2++) {
        dp2[ptr] = std::max(extract(i, j - 1, j2), extract(i, j, j2 - 1));
        std::pair<int,int> sec = extract(i, j - 1, j2 - 1);
        if(0 <= ptrglobal)
          sec = std::max(sec, global[ptrglobal]);

        if(a[j] == b[j2] && dp2[ptr].first <= sec.first) {
          dp2[ptr] = sec;
          dp2[ptr].first++;
          last[startid[i] + ptr] = dp2[ptr].second;
          dp2[ptr].second = startid[i] + ptr;
          if(smax < dp2[ptr].first) {
            smax = dp2[ptr].first;
            smaxid = dp2[ptr].second;
          }
        }
        global[i] = std::max(global[i], dp2[ptr]);
        ptr++;
      }
    std::swap(dp, dp2);
    for(int j = 0; j < dp2.size(); j++)
      dp2[j] = {0, 0};
  }

  std::vector<int> sol;
  int node = smaxid, ptr = k - 1;

  while(0 < smax) {
    while(node < startid[ptr])
      ptr--;
    sol.push_back(a[startx[ptr] + (node - startid[ptr]) / (stopy[ptr] - starty[ptr] + 1)]);

    node = last[node];
    smax--;
  }

  std::reverse(sol.begin(), sol.end());
  return sol;
}
