#include <iostream>
#include <vector>
#include <algorithm>
#include <cassert>
#include <cmath>
#include <unordered_map>

using ll = long long;
int const nmax = 1000000;

class FenwickTree{
private:
  std::vector<int>aib;
  int n;
public:
  FenwickTree(int n_) {
    n = n_;
    aib.resize(1 + n);
  }
  void update(int pos, int val) {
    for(int x = pos; x <= n; x += (x ^ (x & (x - 1))))
      aib[x] = std::max(aib[x], val);
  }

  int query(int pos) {
    int result = 0;
    for(int x = pos; 0 < x; x = (x & (x - 1)))
      result = std::max(result, aib[x]);
    return result;
  }
};

bool compare(std::pair<int,int> f1, std::pair<int, int> f2) {
  if(f1.first != f2.first)
    return f1.first < f2.first;
  else
    return f2.second < f1.second;
}

int main() {
  std::ios::sync_with_stdio(0);
  std::cin.tie(0);

  int n;
  std::cin >> n;
  std::vector<std::pair<int,int>> v;
  for(int i = 1;i <= n; i++) {
    int a, b;
    std::cin >> a >> b;
    v.push_back({-a - b, a - b});
  }
 
  std::sort(v.begin(), v.end(), compare);
  std::vector<int> aux;
  for(int i = 0; i < v.size(); i++) {
    aux.push_back(v[i].second);
  }
  
  std::sort(aux.begin(), aux.end());
  aux.erase(std::unique(aux.begin(), aux.end()),aux.end());
  
  for(int i = 0; i < v.size(); i++) {
    int x = 0;
    for(int jump = (1 << 20); 0 < jump; jump /= 2)
      if(x + jump < aux.size() && aux[x + jump] <= v[i].second)
        x += jump;
    v[i].second = x + 1;
  }
  
  FenwickTree aib(1 + n);
  for(int i = 0; i < v.size(); i++) {
    int val = aib.query(v[i].second - 1);
    aib.update(v[i].second, val + 1);
  }
  std::cout << aib.query(n) << '\n';
  return 0;
}
