#include "hint.h"

#include <iostream>
#include <limits>
#include <algorithm>

#define F first
#define S second

const int MAXLEN = 18; // 18
const int W = 17;

using namespace std;
using pi = pair<int, int>;
using ll = long long;

vector<bool> genHint(const vector<int>& a, const vector<int>& b, const vector<int>& sol) {
    vector<int> pos_a, pos_b;

    int ptr = 0;
    for (int x : sol) {
        while (a[ptr] != x) ++ptr;
        pos_a.push_back(ptr++);
    }

    ptr = 0;
    for (int x : sol) {
        while (b[ptr] != x) ++ptr;
        pos_b.push_back(ptr++);
    }

    int s = 1, deg = 0;
    vector<pi> help;

    const int k = sol.size();

    while (true) {
        help.clear();
        help.emplace_back(pos_a[0], pos_b[0]);
        for (int i = 1; i < k; ++i) {
            while (i+1 < k && (ll)(pos_a[i+1] - help.back().F) * (pos_b[i+1] - help.back().S) <= s) ++i;
            help.emplace_back(pos_a[i], pos_b[i]);
        }

        if (help.size() > MAXLEN) {
            s *= 2;
            deg++;
        } else {
            vector<bool> res(help.size() * W * 2 + W);
            for (int i = 0; i < help.size(); ++i) {
                for (int x = help[i].F, j = 2*W*i;   x != 0; x >>= 1, ++j) res[j] = x & 1;
                for (int x = help[i].S, j = 2*W*i+W; x != 0; x >>= 1, ++j) res[j] = x & 1;
            }

            for (int x = deg, j = help.size()*2*W; x != 0; x >>= 1, ++j) res[j] = x & 1;

/*
            cerr << "s = " << s << endl;
            cerr << "help: ";
            for (auto [a, b] : help) cerr << " [" << a << ":" << b << "]";
            cerr << endl;
*/

            return res;
        }
    }
}

vector<int> solve(const vector<int>& a, const vector<int>& b, const vector<bool>& hint) {
    int len = (hint.size() - W) / (2 * W);

    auto read_int = [&hint] (int pos) {
        int res = 0; pos *= W;
        for (int i = 0; i < W; ++i) {
            if (hint[pos+i]) res += 1 << i;
        }
        return res;
    };

    int s = 1 << read_int(2 * len);
    vector<pi> help(len);
    for (int i = 0; i < len; ++i) {
        help[i].F = read_int(2 * i);
        help[i].S = read_int(2 * i + 1);
    }

/*
    cerr << "s = " << s << endl;
    cerr << "help: ";
    for (auto [a, b] : help) cerr << " [" << a << ":" << b << "]";
    cerr << endl;
*/

    vector<int> res;

    for (int i = 0; i + 1 < len; ++i) {
        res.push_back(a[help[i].F]);
        if ((ll) (help[i+1].F - help[i].F) * (help[i+1].S - help[i].S) > s) continue;

        int dx = help[i].F + 1;
        int dy = help[i].S + 1;

        vector dp(help[i+1].F - help[i].F - 1, vector<int>(help[i+1].S - help[i].S - 1, 0));

        for (int x = help[i].F+1; x < help[i+1].F; ++x)
        for (int y = help[i].S+1; y < help[i+1].S; ++y) {
            int cx = x - dx, cy = y - dy;
            if (cx != 0) dp[cx][cy] = max(dp[cx][cy], dp[cx-1][cy]);
            if (cy != 0) dp[cx][cy] = max(dp[cx][cy], dp[cx][cy-1]);

            if (a[x] == b[y]) {
                if (cx && cy) dp[cx][cy] = max(dp[cx][cy], 1 + dp[cx-1][cy-1]);
                else dp[cx][cy] = 1;
            }
        }

/*
        cerr << "u:"; for (int x = help[i].F+1; x < help[i+1].F; ++x) cout << ' ' << a[x]; cout << endl;
        cerr << "v:"; for (int x = help[i].S+1; x < help[i+1].S; ++x) cout << ' ' << b[x]; cout << endl;
*/

        vector<int> seq;

        int px = help[i+1].F - dx - 1, py = help[i+1].S - dy - 1;
        while (px >= 0 && py >= 0) {
            if (a[px+dx] == b[py+dy]) {
                seq.push_back(a[px+dx]);
                px--;
                dy--;
            } else {
                if (!px) py--;
                else if (!py) px--;
                else if (dp[px-1][py] > dp[px][py-1]) --px;
                else --py;
            }
        }

        reverse(seq.begin(), seq.end());
        for (int x : seq) res.push_back(x);
    }

    res.push_back(a[help.back().F]);

    return res;
}