#include "hint.h"

#include <iostream>
#include <limits>
#include <algorithm>
#include <queue>
#include <iomanip>
#include <cmath>
#include <cassert>

#define F first
#define S second

const int MAXLEN = 130; // 37
const int W = 18;

using namespace std;
using pi = pair<int, int>;
using ll = long long;

const int V = 4e5+228;

struct Node {
    int par;
    int c;
    int go[2];

    Node() :  c(1.0), par(-1) {
        go[0] = go[1] = -1;
    }
} nd[V];

struct cmp {
    bool operator() (int i, int j) { return nd[i].c == nd[j].c ? i < j : nd[i].c > nd[j].c; }
};

int vc = 0;

int build_huffman(int n, int k) {
    vc = 0;
    for (int i = 0; i < 2*n + 7; ++i) {
        nd[i].c = 1.0;
        nd[i].par = -1;
        nd[i].go[0] = nd[i].go[1] = -1;
    }

    priority_queue<int, vector<int>, cmp> q;
    double aprox = (double) n / k;
    for (int i = 0; i <= n; ++i) {
        nd[vc].c = 1000 * (i ? min(i/aprox, aprox/i) : min(1/aprox, aprox/1));
        // cout << i << " => " << fixed << setprecision(4) << nd[i].c  << endl;
        q.push(vc++);
    }

    while (q.size() > 1) {
        int a = q.top(); q.pop();
        int b = q.top(); q.pop();
        nd[vc].go[0] = a;
        nd[vc].go[1] = b;
        nd[vc].c = nd[a].c + nd[b].c;
        nd[a].par = vc;
        nd[b].par = vc;

        // cout << "vc=" << vc << " => " << a << " : " << b << endl;

        q.push(vc++);
    }

    return q.top();
}

void write_int(vector<bool>& hint, int x, int root) {

    // cout << "write " << x << endl;

    vector<bool> path;
    int v = x;
    while (v != root) {
        int p = nd[v].par;
        path.push_back(nd[p].go[1] == v);
        v = p;
    }

    reverse(path.begin(), path.end());
    for (bool x : path) hint.push_back(x);
}

vector<bool> genHint(const vector<int>& a, const vector<int>& b, const vector<int>& sol) {
    vector<int> pos_a, pos_b;

    int ptr = 0;
    for (int x : sol) {
        while (a[ptr] != x) ++ptr;
        pos_a.push_back(ptr++);
    }

    ptr = 0;
    for (int x : sol) {
        while (b[ptr] != x) ++ptr;
        pos_b.push_back(ptr++);
    }

    int s = 1, deg = 0;
    vector<pi> help;

    const int k = sol.size();
    const int n = b.size();


    if (k < MAXLEN) {
        vector<bool> res(W*k+1, 0);
        for (int i = 0; i < k; ++i) {
            for (int j = W*i+1, x = sol[i]; x != 0; x >>= 1, ++j) res[j] = x & 1;
        }
        return res;
    } else {
        int root = build_huffman(n, MAXLEN);

        int pos = 0, t = 0;
        vector<bool> res;

        res.push_back(1);

        int prv = 0;
        for (int i = MAXLEN; i > 0; i--) {
            while (pos_a[t] < pos) ++t;
            write_int(res, pos_b[t] - prv, root);
            prv = pos_b[t];
            pos += (a.size() - pos) / i;
        }

        return res;
    }
}

vector<int> solve(const vector<int>& a, const vector<int>& b, const vector<bool>& hint) {
    vector<int> res;

    if (hint[0] == 0) {
        auto read_int = [&hint] (int pos) {
            int res = 0; pos = pos * W + 1;
            for (int i = 0; i < W; ++i) {
                if (hint[pos+i]) res += 1 << i;
            }
            return res;
        };
        for (int j = 0; j < hint.size()/W; ++j) res.push_back(read_int(j));
        return res;
    }

    int root = build_huffman(b.size(), MAXLEN);

    const int m = b.size();

    int read_pos = 1;
    auto read_int = [&hint, &read_pos, root, m] () {
        int v = root;
        while (v > m) {
            assert(read_pos < hint.size());
            int x = hint[read_pos++];
            v = nd[v].go[x ? 1 : 0];
        }
        return v;
    };

    vector<pi> help;

    int pos = 0, j = 0;
    for (int i = MAXLEN; i > 0; i--) {
        j += read_int();
        help.emplace_back(pos, j);
        pos += (a.size() - pos) / i;
    }

    help.emplace_back((int)a.size(), (int)b.size());

/*
    for (int i = 0; i <= b.size(); ++i) {
        vector<bool> see;
        write_int(see, i, root);
        for (int x : see) cout << x;
        int read_pos = 0;

        int v = root;
        while (v > m) {
            int x = see[read_pos++];
            v = nd[v].go[x];
        }

        cout << " => " << v << endl;
    }
*/

    vector<vector<int>> dp(a.size()/MAXLEN + 3);

    for (int i = 0; i < MAXLEN; ++i) {
        int dx = help[i].F;
        int dy = help[i].S;

        int nx = help[i+1].F - dx;
        int ny = help[i+1].S - dy;

        if (nx == 0 || ny == 0) continue;

        for (int i = nx; i--; ) dp[i].assign(ny, 0);

        for (int x = dx; x < help[i+1].F; ++x) {
            for (int y = dy; y < help[i+1].S; ++y) {
                int cx = x - dx, cy = y - dy;
                if (cx != 0) dp[cx][cy] = max(dp[cx][cy], dp[cx-1][cy]);
                if (cy != 0) dp[cx][cy] = max(dp[cx][cy], dp[cx][cy-1]);

                if (a[x] == b[y]) {
                    if (cx && cy) dp[cx][cy] = max(dp[cx][cy], 1 + dp[cx-1][cy-1]);
                    else dp[cx][cy] = 1;
                }
            }
        }

        vector<int> seq;

        int px = help[i+1].F - dx - 1, py = help[i+1].S - dy - 1;
        while (px >= 0 && py >= 0) {
            if (a[px+dx] == b[py+dy]) {
                seq.push_back(a[px+dx]);
                px--;
                py--;
            } else {
                if (!px) py--;
                else if (!py) px--;
                else if (dp[px-1][py] > dp[px][py-1]) --px;
                else --py;
            }
        }

        reverse(seq.begin(), seq.end());
        for (int x : seq) res.push_back(x);
    }

    return res;
}