#include <cmath>
#include <vector>
#include <iomanip>
#include <iostream>
#include <algorithm>

using namespace std;

#ifdef DEBUG

#else
#include "hint.h"
#endif

const int K = 447;
const int KK = 18;

vector<bool> genHint(const vector<int>& a, const vector<int>& b, const vector<int>& sol) {
  int n = a.size(), m = b.size(), k = sol.size();
  vector<int> ids1(k);
  vector<int> ids2(k);
  int j = 0;
  for (int i = 0; i < n; ++i) {
    if (j < k && sol[j] == a[i]) {
      ids1[j] = i;
      ++j;
    }
  }
  j = 0;
  for (int i = 0; i < m; ++i) {
    if (j < k && sol[j] == b[i]) {
      ids2[j] = i;
      ++j;
    }
  }
  vector<int> couple(n, m);
  for (int i = 0; i < k; ++i)
    couple[ids1[i]] = ids2[i];
  vector<vector<int>> ids(n);
  for (int i = 0; i < n; ++i)
    ids[a[i]].emplace_back(i);
  vector<int> numbers;
  for (int i = 0; i < n; ++i) {
    vector<int> tmp(ids[i].size());
    for (int j = 0; j < (int) ids[i].size(); ++j)
      tmp[j] = couple[ids[i][j]];
    for (int j = (int) ids[i].size() - 1; j >= 0; --j)
      tmp[j] = min(tmp[j], tmp[j + 1]);
    for (int j = K; j < (int) ids[i].size(); j += K)
      numbers.emplace_back(tmp[j]);
  }
  vector<bool> hint;
  hint.reserve(KK * numbers.size());
  for (int x : numbers) {
    for (int j = 0; j < KK; ++j)
      hint.emplace_back((x >> j) & 1);
  }
  return hint;
}

const int MAXN = 200'200;

pair<int, int> fenw[MAXN];

void upd(int i, pair<int, int> val) {
  for (; i < MAXN; i = (i | (i + 1)))
    fenw[i] = max(fenw[i], val);
}

pair<int, int> get(int i) {
  pair<int, int> ans(0, 0);
  for (; i >= 0; i = (i & (i + 1)) - 1)
    ans = max(ans, fenw[i]);
  return ans;
}

vector<int> solve(const vector<int>& a, const vector<int>& b, const vector<bool>& hint) {
  int n = a.size(), m = b.size();
  int hint_id = 0;
  vector<vector<int>> ids(n);
  for (int i = 0; i < n; ++i)
    ids[a[i]].emplace_back(i);
  vector<int> to_add;
  for (int i = 0; i < n; ++i) {
    for (int j = K; j < (int) ids[i].size(); j += K) {
      int cur = 0;
      for (int l = 0; l < KK; ++l)
        cur += int(hint[hint_id + l]) << l;
      hint_id += KK;
      to_add.emplace_back(cur);
    }
  }
  sort(to_add.begin(), to_add.end());
  int j = 0;
  fill(fenw, fenw + n, make_pair(0, 0));
  vector<int> coeff(n);
  vector<int> dp(n);
  vector<int> par(n, -1);
  for (int i = 0; i < m; ++i) {
    while (j < (int) to_add.size() && to_add[j] == i) {
      ++coeff[b[i]];
      ++j;
    }
    if (ids[b[i]].empty())
      continue;

    for (int l = min((int) ids[b[i]].size(), coeff[b[i]] * K + K) - 1; l >= coeff[b[i]] * K; --l) {
      int ii = ids[b[i]][l];
      auto cur = get(ii - 1);
      dp[ii] = cur.first + 1;
      par[ii] = cur.second - 1;
      upd(ii, make_pair(dp[i], ii + 1));
    }
  }
  int mx = 0;
  for (int i = 0; i < n; ++i) {
    if (dp[i] > dp[mx])
      mx = i;
  }
  if (!dp[mx])
    return {};
  vector<int> ans;
  while (mx != -1) {
    ans.emplace_back(a[mx]);
    mx = par[mx];
  }
  reverse(ans.begin(), ans.end());
  return ans;
}


#ifdef DEBUG
int main() {
#ifdef DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#else
  ios::sync_with_stdio(false);
  cin.tie(0);
  cout.tie(0);
#endif

  static int n, m, k;
  static std::vector<int> a, b, sol, maybeSol;
  static std::vector<bool> hint;

  std::cin >> n >> m;

  a.resize(n);
  b.resize(m);

  for (int& num : a)
  {
    std::cin >> num;
  }

  for (int& num : b)
  {
    std::cin >> num;
  }

  std::cin >> k;

  sol.resize(k);

  for (int& num : sol)
  {
    std::cin >> num;
  }

  hint = genHint(a, b, sol);
  maybeSol = solve(a, b, hint);

  std::cout << hint.size() << std::endl;
  std::cout << maybeSol.size() << std::endl;

  for (int i = 0; i < (int) maybeSol.size(); ++i)
  {
    if (i > 0) std::cout << " ";
    std::cout << maybeSol[i];
  }
  std::cout << std::endl;

  return 0;
}
#endif