#include "hint.h"

#include <bits/stdc++.h>

using namespace std;


const int MAXN = 200000;
// const int K = 5715;
const int K = 1000;


vector<bool> genHint(const vector<int>& a, const vector<int>& b, const vector<int>& sol) {
	int i = 0;
	int j = 0;
	vector<pair<int, int>> sol_idxes;
	for(int x: sol) {
		while(a[i] != x) {
			i++;
		}
		while(b[j] != x) {
			j++;
		}
		sol_idxes.emplace_back(i, j);
		i++;
		j++;
	}

	vector<int> separators;
	for(int i = K; i < a.size(); i += K) {
		auto it = lower_bound(sol_idxes.begin(), sol_idxes.end(), make_pair(i, 0));
		separators.push_back(it == sol_idxes.end() ? b.size() : it->second);
	}

	vector<bool> hint;
	for(int x: separators) {
		for(int j = 0; j < 18; j++) {
			hint.push_back((x >> j) & 1);
		}
	}

	return hint;
}


int dp1[MAXN];
int dp2[MAXN];


void hirshberg(const vector<int>& a, const vector<int>& b, int al, int ar, int bl, int br, vector<int>& sol) {
	if(bl == br) {
		return;
	}

	if(ar - al == 1) {
		for(int i = bl; i < br; i++) {
			if(b[i] == a[al]) {
				sol.push_back(a[al]);
				return;
			}
		}
		return;
	}

	int am = (al + ar) / 2;

	fill(dp1, dp1 + br - bl, 0);
	for(int i = 0; i < am - al; i++) {
		int old = 0;
		for(int j = 0; j < br - bl; j++) {
			int prev_old = old;
			old = dp1[j];
			if(a[al + i] == b[bl + j]) {
				dp1[j] = prev_old + 1;
			} else {
				dp1[j] = max(dp1[j], j == 0 ? 0 : dp1[j - 1]);
			}
		}
	}

	fill(dp2, dp2 + br - bl, 0);
	for(int i = ar - am - 1; i >= 0; i--) {
		int old = 0;
		for(int j = br - bl - 1; j >= 0; j--) {
			int prev_old = old;
			old = dp2[j];
			if(a[am + i] == b[bl + j]) {
				dp2[j] = prev_old + 1;
			} else {
				dp2[j] = max(dp2[j], j + 1 == br - bl ? 0 : dp2[j + 1]);
			}
		}
	}

	int best_sum = -1;
	int best_j = -1;
	for(int j = 0; j <= br - bl; j++) {
		int sum = (j == 0 ? 0 : dp1[j - 1]) + (j == br - bl ? 0 : dp2[j]);
		if(sum > best_sum) {
			best_sum = sum;
			best_j = j;
		}
	}
	int bm = bl + best_j;

	if(best_sum == 0) {
		return;
	}

	hirshberg(a, b, al, am, bl, bm, sol);
	hirshberg(a, b, am, ar, bm, br, sol);
}


vector<int> solve(const vector<int>& a, const vector<int>& b, const vector<bool>& hint) {
	vector<int> separators;
	for(int i = 0; i < hint.size(); i += 18) {
		int x = 0;
		for(int j = 0; j < 18; j++) {
			x |= (int)hint[i + j] << j;
		}
		separators.push_back(x);
	}


	vector<int> sol;

	for(int i = 0; i < a.size(); i += K) {
		int al = i;
		int ar = min(i + K, (int)a.size());
		int bl = i == 0 ? 0 : separators[i / K - 1];
		int br = i / K == separators.size() ? b.size() : separators[i / K];
		if(bl < br) {
			hirshberg(a, b, al, ar, bl, br, sol);
		}
	}

	return sol;
}


void unit_test() {
}
