#include "hint.h"

#include <bits/stdc++.h>

using namespace std;


vector<bool> rle_encode(const vector<bool>& in) {
	if(in.empty()) {
		return in;
	}
	vector<bool> out;
	out.push_back(1);
	out.push_back(in[0]);
	int l = 0;
	while(l < in.size()) {
		int r = l;
		while(r < in.size() && in[r] == in[l]) {
			r++;
		}
		int c = r - l;
		if(c == 1) {
			out.push_back(0);
		} else if(c == 2) {
			out.push_back(1);
			out.push_back(0);
		} else if(c == 3) {
			out.push_back(1);
			out.push_back(1);
			out.push_back(0);
		} else {
			out.push_back(1);
			out.push_back(1);
			out.push_back(1);
			for(int j = 0; j < 18; j++) {
				out.push_back((c >> j) & 1);
			}
		}
		l = r;
	}
	if(out.size() > in.size() + 1) {
		out = in;
		out.insert(out.begin(), 0);
	}
	return out;
}
vector<bool> rle_decode(const vector<bool>& in) {
	if(in.empty()) {
		return in;
	}
	vector<bool> out;
	if(in[0] == 1) {
		bool cur_val = in[1];
		int i = 2;
		while(i < in.size()) {
			int c;
			if(in[i++] == 0) {
				c = 1;
			} else {
				if(in[i++] == 0) {
					c = 2;
				} else {
					if(in[i++] == 0) {
						c = 3;
					} else {
						c = 0;
						for(int j = 0; j < 18; j++) {
							c |= (int)in[i++] << j;
						}
					}
				}
			}
			for(int j = 0; j < c; j++) {
				out.push_back(cur_val);
			}
			cur_val = !cur_val;
		}
	} else {
		out.insert(out.end(), in.begin() + 1, in.end());
	}
	return out;
}
vector<bool> mtf_encode(const vector<bool>& in) {
	vector<bool> out;
	bool prev = false;
	for(bool x: in) {
		out.push_back(x ^ prev);
		prev = x;
	}
	return out;
}
vector<bool> mtf_decode(const vector<bool>& in) {
	vector<bool> out;
	bool prev = false;
	for(bool x: in) {
		out.push_back(x ^ prev);
		prev = out.back();
	}
	return out;
}



const int MAXN = 200000;
const int K = 2300;


void encode_numbers(const vector<int>& nums, int l, int r, int xl, int xr, vector<bool>& hint) {
	if(l == r || xl >= xr) {
		return;
	}

	int m = (l + r) / 2;

	int bitlen = 0;
	while((1 << bitlen) < xr - xl) {
		bitlen++;
	}

	int val = nums[m] - xl;
	for(int i = 0; i < bitlen; i++) {
		hint.push_back((val >> i) & 1);
	}

	encode_numbers(nums, l, m, xl, nums[m] + 1, hint);
	encode_numbers(nums, m + 1, r, nums[m], xr, hint);
}

void decode_numbers(vector<int>& nums, int l, int r, int xl, int xr, const vector<bool>& hint, int& hint_ptr) {
	if(l == r || xl >= xr) {
		return;
	}

	int m = (l + r) / 2;

	int bitlen = 0;
	while((1 << bitlen) < xr - xl) {
		bitlen++;
	}

	int num = 0;
	for(int i = 0; i < bitlen; i++) {
		num |= (int)hint[hint_ptr++] << i;
	}
	num += xl;
	nums[m] = num;

	decode_numbers(nums, l, m, xl, num + 1, hint, hint_ptr);
	decode_numbers(nums, m + 1, r, num, xr, hint, hint_ptr);
}


vector<bool> genHint(const vector<int>& a, const vector<int>& b, const vector<int>& sol) {
	int i = 0;
	int j = 0;
	vector<pair<int, int>> sol_idxes;
	for(int x: sol) {
		while(a[i] != x) {
			i++;
		}
		while(b[j] != x) {
			j++;
		}
		sol_idxes.emplace_back(i, j);
		i++;
		j++;
	}

	vector<int> separators;
	for(int i = K; i < a.size(); i += K) {
		auto it = lower_bound(sol_idxes.begin(), sol_idxes.end(), make_pair(i, 0));
		separators.push_back(it == sol_idxes.end() ? b.size() : it->second);
	}

	vector<bool> hint;
	encode_numbers(separators, 0, separators.size(), 0, b.size() + 1, hint);
	return hint;
}


const int MAXM = 400;

int dp1[MAXN];
int dp2[MAXN];
int dp[MAXM][MAXN];


void hirshberg(const vector<int>& a, const vector<int>& b, int al, int ar, int bl, int br, vector<int>& sol) {
	if(bl == br) {
		return;
	}

	if(ar - al <= MAXM) {
		for(int i = 0; i < ar - al; i++) {
			for(int j = 0; j < br - bl; j++) {
				if(a[al + i] == b[bl + j]) {
					dp[i][j] = (i == 0 || j == 0 ? 0 : dp[i - 1][j - 1]) + 1;
				} else {
					dp[i][j] = max(i == 0 ? 0 : dp[i - 1][j], j == 0 ? 0 : dp[i][j - 1]);
				}
			}
		}
		int ai = ar - al - 1;
		int bi = br - bl - 1;
		if(dp[ai][bi] > 0) {
			sol.resize(sol.size() + dp[ai][bi]);
			int pos = sol.size();
			while(ai >= 0 && bi >= 0 && dp[ai][bi] > 0) {
				if(a[al + ai] == b[bl + bi]) {
					sol[--pos] = a[al + ai];
					ai--;
					bi--;
				} else {
					if(ai > 0 && dp[ai - 1][bi] == dp[ai][bi]) {
						ai--;
					} else {
						bi--;
					}
				}
			}
		}
		return;
	}

	int am = (al + ar) / 2;

	fill(dp1, dp1 + br - bl, 0);
	for(int i = 0; i < am - al; i++) {
		int prev_old = 0;
		for(int j = 0; j < br - bl; j++) {
			int old = dp1[j];
			if(a[al + i] == b[bl + j]) {
				dp1[j] = prev_old + 1;
			} else {
				dp1[j] = max(dp1[j], j == 0 ? 0 : dp1[j - 1]);
			}
			prev_old = old;
		}
	}

	fill(dp2, dp2 + br - bl, 0);
	for(int i = ar - am - 1; i >= 0; i--) {
		int prev_old = 0;
		for(int j = br - bl - 1; j >= 0; j--) {
			int old = dp2[j];
			if(a[am + i] == b[bl + j]) {
				dp2[j] = prev_old + 1;
			} else {
				dp2[j] = max(dp2[j], j + 1 == br - bl ? 0 : dp2[j + 1]);
			}
			prev_old = old;
		}
	}

	int best_sum = -1;
	int best_j = -1;
	for(int j = 0; j <= br - bl; j++) {
		int sum = (j == 0 ? 0 : dp1[j - 1]) + (j == br - bl ? 0 : dp2[j]);
		if(sum > best_sum) {
			best_sum = sum;
			best_j = j;
		}
	}
	int bm = bl + best_j;

	if(best_sum == 0) {
		return;
	}

	hirshberg(a, b, al, am, bl, bm, sol);
	hirshberg(a, b, am, ar, bm, br, sol);
}


vector<int> solve(const vector<int>& a, const vector<int>& b, const vector<bool>& hint) {
	vector<int> separators;
	for(int i = K; i < a.size(); i += K) {
		separators.push_back(-1);
	}

	int hint_ptr = 0;
	decode_numbers(separators, 0, separators.size(), 0, b.size() + 1, hint, hint_ptr);


	vector<int> sol;

	for(int i = 0; i < a.size(); i += K) {
		int al = i;
		int ar = min(i + K, (int)a.size());
		int bl = i == 0 ? 0 : separators[i / K - 1];
		int br = i / K == separators.size() ? b.size() : separators[i / K];
		if(bl < br) {
			hirshberg(a, b, al, ar, bl, br, sol);
		}
	}

	return sol;
}


void unit_test() {
	vector<bool> vec{1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1};
	assert(rle_decode(rle_encode(vec)) == vec);
	assert(mtf_decode(mtf_encode(vec)) == vec);
}
