#include <algorithm>
#include <iostream>
#include <utility>
#include <vector>
#include <cmath>
#include <queue>
#define endl '\n'
using namespace std;

const int MAXN = 1000000;

struct edge
{
    int adj, rev, cap, flow;

    edge(int adj, int rev, int cap, int flow)
    {
        this -> adj = adj;
        this -> rev = rev;
        this -> cap = cap;
        this -> flow = flow;
    }
};

int n;
int c[MAXN + 5];
int sz[MAXN + 5];
int idx[2 * MAXN + 5];
int lev[2 * MAXN + 5];
pair<int, int> p[MAXN + 5];
vector<edge> G[2 * MAXN + 5];

int comp(int u)
{
    if (c[u] != c[c[u]]) c[u] = comp(c[u]);
    return c[u];
}

void unite(int u, int v)
{
    u = comp(u);
    v = comp(v);

    if (sz[v] > sz[u]) swap(u, v);
    c[v] = u;
    sz[u] += sz[v];
}

void add_edge(int u, int v)
{
    G[u].emplace_back(v, G[v].size(), 1, 0);
    G[v].emplace_back(u, G[u].size() - 1, 0, 0);
}

bool BFS()
{
    for (int i = 0; i <= 2 * n + 1; ++ i)
    {
        idx[i] = lev[i] = 0;
    }

    queue<int> q;
    q.push(0);
    lev[0] = 1;

    while (!q.empty())
    {
        int u = q.front();
        q.pop();

        if (u == 2 * n + 1) return true;

        for (auto e : G[u])
        {
            if (lev[e.adj] == 0 and e.flow < e.cap)
            {
                lev[e.adj] = lev[u] + 1;
                q.push(e.adj);
            }
        }
    }

    return false;
}

int DFS(int u, int f)
{
    if (u == 2 * n + 1) return f;

    while (idx[u] < G[u].size())
    {
        int v = G[u][idx[u]].adj;
        if (lev[v] == lev[u] + 1 and G[u][idx[u]].flow < G[u][idx[u]].cap)
        {
            int flow = DFS(v, min(f, G[u][idx[u]].cap - G[u][idx[u]].flow));
            if (flow != 0)
            {
                G[u][idx[u]].flow = flow;
                G[v][G[u][idx[u]].rev].flow -= flow;
                return flow;
            }
        }
        idx[u]++;
    }

    return 0;
}

int main()
{
    ios :: sync_with_stdio(false);
    cin.tie(NULL); cout.tie(NULL);

    cin >> n;

    for (int i = 1; i <= n; ++ i)
    {
        cin >> p[i].first >> p[i].second;
        c[i] = i;
        sz[i] = 1;
    }

    sort(p + 1, p + n + 1);

    for (int i = 1; i <= n; ++ i)
    {
        add_edge(0, 2 * i - 1);
    }

    for (int j = 1; j <= n; ++ j)
    {
        for (int i = 1; i < j; ++ i)
        {
            if (p[j].first - p[i].first >= abs(p[j].second - p[i].second) and comp(i) != comp(j))
            {
                add_edge(2 * i - 1, 2 * j);
                unite(i, j);
            }
        }
    }

    for (int i = 1; i <= n; ++ i)
    {
        add_edge(2 * i, 2 * n + 1);
    }

    int flow = 0;
    while (BFS())
    {
        while (true)
        {
            int f = DFS(0, MAXN + 5);
            if (f == 0) break;
            else flow += f;
        }
    }

    cout << n - flow << endl;

    return 0;
}
